use crossterm::event::KeyModifiers;
use derive_setters::Setters;
use ratatui::{
    layout::{
        Constraint,
        Direction,
        Layout,
        Rect
    },
    style::{Color, Style, Stylize},
    text::{Line, Span, Text},
    widgets::{
        Block,
        Borders,
        Clear,
        List,
        ListItem,
        ListState,
        Paragraph,
        Widget,
        Wrap
    },
    Frame,
    crossterm::event::{
        self,
        Event,
        KeyCode,
        KeyEvent,
        KeyEventKind
    }
};
use tui_textarea::{Key, Input, TextArea};

use crate::{client::Client, db::AppState, message::{Message, MessageKind}, player::Player};

const HELP: &str = r#"bindings:
h - show help
q - quit
1 - create player
2 - create lobby
3 - send player to server
4 - send lobby to server"#;

// popup things
#[derive(Debug, Default, Setters)]
struct Popup<'a> {
    #[setters(into)]
    title: Line<'a>,
    #[setters(into)]
    content: Text<'a>,
    border_style: Style,
    title_style: Style,
    style: Style,
}

// popup things
impl Widget for Popup<'_> {
    fn render(self, area: Rect, buf: &mut ratatui::prelude::Buffer)
        where
            Self: Sized {
        Clear.render(area, buf);

        let block = Block::new()
            .title(self.title)
            .title_style(self.title_style)
            .borders(Borders::ALL)
            .border_style(self.border_style);

        Paragraph::new(self.content)
            .wrap(Wrap { trim: true })
            .style(self.style)
            .block(block)
            .render(area, buf);
    }
}

async fn handle_key_event(mut client: &mut Client<'_>, key_event: KeyEvent) {
    match &client.popup_user_input {
        true => {
            match key_event.into() {   
                Input {key: Key::Esc, ..} => client.popup_user_input = false,
                input => {
                    client.textarea.input(input);
                },
            }
        },
        false => {
            match &client.popup {
                false => {
                    match key_event.code {
                        // quit
                        KeyCode::Char('q') => Client::exit(client),
                        KeyCode::Char('h') => {
                            client.popup_title = "help".to_string();
                            client.popup_content = HELP.to_string(); 
                            client.popup = true;
                        },
                        // player creation
                        KeyCode::Char('1') => {
                            if client.user != None {
                                client.popup_title = "Error".to_string();
                                client.popup_content = "user already_created".to_string();
                                client.popup = true;
                            } else {
                                let addr = client.addr.clone();
                                let path = "/create/player";
                                let message = Message::new(
                                    "",
                                    MessageKind::CreatePlayer,
                                    ""
                                ).unwrap();

                                Client::send(&mut client, message, &addr, &path).await.unwrap();
                                client.user_name = client.user.clone().unwrap();
                            }
                        },
                        // lobby creation
                        KeyCode::Char('2') => {
                            if client.lobby != None || client.user == None {
                                client.popup_title = "Error".to_string();
                                client.popup_content = "lobby already_created or player not yet created".to_string();
                                client.popup = true;
                            } else {
                                let addr = client.addr.clone();
                                let path = "/create/lobby";
                                let message = Message::new(
                                    "",
                                    MessageKind::CreateLobby,
                                    &client.user_name,
                                ).unwrap();

                                Client::send(&mut client, message, &addr, &path).await.unwrap();
                                client.lobby_id = client.lobby.clone().unwrap();
                            }
                        },
                        // popup for lobby search
                        KeyCode::Char('3') => {
                            client.popup_user_input = true;
                        }
                        _ => (),
                    }
                }
                true => {
                    match key_event.code {
                        _ => client.popup = false,
                    }
                }
            }
        }
    }
}

pub async fn handle_events(app: &mut Client<'_>) -> anyhow::Result<()> {
    match event::read()? {
        Event::Key(key) if key.kind == KeyEventKind::Press => {
            handle_key_event(app, key).await
        }
        _ => ()
    }

    Ok(())
}

pub fn ui(frame: &mut Frame, app: &mut Client) {
    // main ui
    let chunks = Layout::default()
        .direction(Direction::Vertical)
        .constraints([
            Constraint::Min(3),
            Constraint::Min(1),
        ])
        .split(frame.area());

    let instructions = Line::from(vec![
        " 1".blue().bold(),
        " create player ".into(),
        "2".blue().bold(),
        " create lobby ".into(),
        "3".blue().bold(),
        " search lobby ".into(),
        "<Q>".blue().bold(),
        " quit ".into(),
    ]);

    let infos = Text::from(vec![
        Line::from(vec![
            "connected to: ".into(),
            app.addr.clone().yellow(),
        ]),
        Line::from(vec![
            "username: ".into(),
            app.user_name.clone().yellow(),
        ]),
        Line::from(vec![
            "lobby: ".into(),
            app.lobby_id.clone().yellow(),
        ]),
    ]);

    let top_block_title = Line::from(" durak tui test fre ".bold());
    let top_block = Block::default()
        .title(top_block_title.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(Paragraph::new(infos).block(top_block), chunks[0]);

    let server_message = Text::from(vec![
        Line::from(vec![
            "message: ".into(),
            app.response.clone().blue(),
        ])
    ]);

    let btm_block_title = Line::from(" messaggini carini :3 ".bold());
    let btm_block = Block::default()
        .title(btm_block_title.centered())
        .title_bottom(instructions.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(
        Paragraph::new(server_message)
            .wrap(Wrap { trim: true, })
            .block(btm_block),
        chunks[1],
    );

    // popup things
    let popup_area = Rect {
        x: frame.area().width / 4,
        y: frame.area().height / 3,
        width: frame.area().width / 2,
        height: frame.area().height / 3,
    };

    if app.popup == true {
        let popup = Popup::default()
            .title(app.popup_title.clone())
            .content(app.popup_content.clone())
            .style(Style::new().yellow())
            .title_style(Style::new().white().bold())
            .border_style(Style::new().red());
        frame.render_widget(popup, popup_area);
    }

    
    let mut textarea = TextArea::default();
    textarea.set_block(
        Block::default()
            .borders(Borders::ALL)
            .border_style(Style::default().fg(Color::LightBlue))
            .title("Crossterm Popup Example"),
    );

    let area = Rect {
        width: 40,
        height: 5,
        x: 5,
        y: 5,
    };

    textarea.set_style(Style::default().fg(Color::Yellow));
    textarea.set_placeholder_style(Style::default());
    app.textarea = textarea;

    if app.popup_user_input == true {
        frame.render_widget(&app.textarea, area);
    }
}
